cwlVersion: v1.0
class: CommandLineTool


requirements:
- class: InlineJavascriptRequirement
  expressionLib:
  - var default_output_filename = function() {
        if (inputs.output_filename == ""){
          let ext = ".gff";
          let root = inputs.islands_file.basename.split('.').slice(0,-1).join('.');
          return (root == "")?inputs.islands_file.basename+ext:root+ext;
        } else {
          return inputs.output_filename;
        }
    };

hints:
- class: DockerRequirement
  dockerPull: biowardrobe2/rose:v0.0.2


inputs:

  islands_file:
    type: File
    inputBinding:
      position: 5
    doc: |
      Input tab-delimited file (output from iaintersect.cwl or macs2-callpeak-biowardrobe-only.cwl tool)

  output_filename:
    type: string?
    inputBinding:
      position: 6
      valueFrom: $(default_output_filename())
    default: ""

  islands_control_file:
    type: File?
    inputBinding:
      position: 7
    doc: |
      Control tab-delimited file (output from iaintersect.cwl or macs2-callpeak-biowardrobe-only.cwl tool)


outputs:
  gff_file:
    type: File
    outputBinding:
      glob: $(default_output_filename())


baseCommand: ['makegff']


$namespaces:
  s: http://schema.org/

$schemas:
- http://schema.org/docs/schema_org_rdfa.html

s:mainEntity:
  $import: ./metadata/makegff-metadata.yaml

s:name: "makegff"
s:downloadUrl: https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/makegff.cwl
s:codeRepository: https://github.com/Barski-lab/workflows
s:license: http://www.apache.org/licenses/LICENSE-2.0

s:isPartOf:
  class: s:CreativeWork
  s:name: Common Workflow Language
  s:url: http://commonwl.org/

s:creator:
- class: s:Organization
  s:legalName: "Cincinnati Children's Hospital Medical Center"
  s:location:
  - class: s:PostalAddress
    s:addressCountry: "USA"
    s:addressLocality: "Cincinnati"
    s:addressRegion: "OH"
    s:postalCode: "45229"
    s:streetAddress: "3333 Burnet Ave"
    s:telephone: "+1(513)636-4200"
  s:logo: "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png"
  s:department:
  - class: s:Organization
    s:legalName: "Allergy and Immunology"
    s:department:
    - class: s:Organization
      s:legalName: "Barski Research Lab"
      s:member:
      - class: s:Person
        s:name: Michael Kotliar
        s:email: mailto:misha.kotliar@gmail.com
        s:sameAs:
        - id: http://orcid.org/0000-0002-6486-3898

doc: |
  Tool produces GFF output from the file generated by iaintersect.cwl or macs2-callpeak-biowardrobe-only.cwl tool
  Both islands_file and islands_control_file should be produced by the same tool

s:about: |
  Tool produces GFF output from the file generated by iaintersect.cwl or macs2-callpeak-biowardrobe-only.cwl tool
